<?php

//------------------------------------------------------------------------
//
//  WebUserImportPlus Template Class
//
//  Author  : Soushi
//  Version : 0.51
//  Date    : 2009/1/18
//
//------------------------------------------------------------------------

class WUIP_Tpl{
  //------------------------------------------Constant value
  const FILE_EXT = '.tpl'; //Template file extension

  //------------------------------------------Private
  private $TemplatePath = './template';  //Template Path
  private $TplList      = array();       //Template File List
  private $PH           = array();       //Placeholder

  //-----------------------------------------------------
  // Class Constructure
  // 
  // $fPath  : string : Template Path
  // return  : none
  //-----------------------------------------------------
  function WUIP_Tpl($fPath=''){
    $fPath != '' && $this->TemplatePath = $fPath;

    //Load template file list
    $templateList = $this->GetDirList('',$this->TemplatePath,WUIP_Tpl::FILE_EXT);
    foreach($templateList as $val){
      $this->TplList[$val] = false;
    }
  } 

  //-----------------------------------------------------
  // The file list of the specified directory is returned. 
  // 
  // $fPath   : string : Specified directory(for recursive)
  // $fBase   : string : Specified base directory
  // $fExt    : string : A file extention of target fie
  // return   : mixed  : File list(exept file extention=$fExt)/false
  //-----------------------------------------------------
  function GetDirList($fPath='',$fBase,$fExt=''){
    $dir = $fPath=='' ? $fBase : $fBase . '/' . $fPath;
    if( !is_dir( $dir ) ) return false;
    
    $extLen= $fExt != '' ? strlen($fExt) : 0;

    $output = array();
    if( $HDL = opendir( $dir ) ){
      while ( false !== $file = readdir($HDL) ){
	if( $file != "." && $file != ".." ) {
	  if( is_dir( $dir.'/'.$file ) )
	    $output = array_merge($output,$this->GetDirList($fPath.$file.'/',$fBase ,$fExt));
	  elseif($extLen == 0 || substr($file,strlen($file)-$extLen) == $fExt )
	    $output[] = $fPath .substr($file,0,strlen($file)-$extLen);
	}
      }
      closedir( $HDL );
    }
    return $output; 
  }

  //-----------------------------------------------------
  // Show template
  // 
  // $fTP           : string : Template name
  // $fPH           : array  : Placeholder
  // $fAllOrNothing : bool   : When the substitution failure is done, the template becomes invalid. 
  //                           Default=false
  // return         : string : HTML / Error message
  //-----------------------------------------------------
  function Show($fTP,$fPH='',$fAllOrNothing=false){

    //Check template 
    if(! isset($this->TplList[$fTP])){
      return 'Template not found!';
    }

    //Check cache
    if( $this->TplList[$fTP] === false ){
      $filename = $this->TemplatePath .'/'. $fTP . WUIP_Tpl::FILE_EXT;
      $this->TplList[$fTP] = file_get_contents($filename);
    }
    $output = $this->TplList[$fTP];

    //Substitution placeholder
    if( $this->DivPlaceholder($fPH,$bf,$af))
      $output = str_replace($bf,$af,$output);

    //Substitution default placeholder
    if( $this->DivPlaceholder($this->PH,$bf,$af))
      $output = str_replace($bf,$af,$output);

    //Substitution template placeholder
    preg_match_all('/\[\+([tT]):([a-zA-Z0-9-_\/]+)\+\]/',$output,$mt);
    $bf=array();
    $af=array();
    foreach( $mt[2] as $key => $val ){
      if($mt[1][$key] == 't')
	$tmpAf=$this->Show($val,$fPH,false);
      else
	$tmpAf=$this->Show($val,$fPH,true);
      if( $tmpAf != '' ){
	$bf[] = $mt[0][$key];
	$af[] = $tmpAf;
      }
    }
    if(! empty($bf))
      $output = str_replace($bf,$af,$output);

    //Remaining placeholder tag is deleted. 
    $output = preg_replace('/\[\+[a-zA-Z0-9-_:\/]+\+\]/','',$output,-1,$cnf);
    //When the substitution failure is done, the template becomes invalid. 
    if( $fAllOrNothing == true && $cnf != 0 )
      $output = '';

    return $output;
  }

  //-----------------------------------------------------
  // Division placeholder
  // 
  // $fPH    : string  : Placeholder
  // $fBf    : string  : Placeholder tag list
  // $fAf    : string  : Substitution data list
  // return  : bool
  //-----------------------------------------------------
  function DivPlaceholder($fPH,&$fBf,&$fAf){
      if(! is_array($fPH))return false;
      $fBf = array();
      $fAf = array();
      foreach( $fPH as $key => $val){
	$fBf[] = '[+' . $key . '+]';
	$fAf[] = $val;
      }
      return true;
  }

  //-----------------------------------------------------
  // Add default placeholder
  // 
  // $fKey   : string  : Placeholder tag
  // $fVal   : string  : Substitution data
  //    --- or --
  // $fKey   : array   : Placeholder tag & Substitution data 
  // $fVal   : bool    : Call by reference?
  // return  : bool
  //-----------------------------------------------------
  function AddPlaceholder($fKey,$fVal=''){
    if( $fKey == '' ){return false;}

    if(! is_array($fKey) ){
      $this->PH[$fKey] = $fVal;
    }else{
      foreach( $fKey as $key => $val){
	if( $fVal == true)
	  $this->PH[$key] = &$fKey[$key];
	else
	  $this->PH[$key] = $val;
      }
    }
    return true;
  }

  //-----------------------------------------------------
  // Delete default placeholder
  // 
  // $fKey   : string  : Placeholder tag
  // return  : bool
  //-----------------------------------------------------
  function DelPlaceholder($fKey){
    if( $fKey == '' ){return false;}
    unset($this->PH[$fKey]);
    return true;
  }

}
?>
